@@ -12,17 +12,19 @@
 // If not, see <http://creativecommons.org/publicdomain/zero/1.0/>.
 //
 
-use std::error;
-use std::fmt;
+use prelude::*;
+
+use core::fmt;
 
 use blockdata::transaction::Transaction;
 use consensus::encode;
 use util::psbt::raw;
 
 use hashes;
+use util::bip32::ExtendedPubKey;
 
+/// Enum for marking psbt hash error.
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug)]
-/// Enum for marking psbt hash error
 pub enum PsbtHash {
     Ripemd,
     Sha256,
@@ -51,16 +53,16 @@ pub enum Error {
     MustHaveUnsignedTx,
     /// Signals that there are no more key-value pairs in a key-value map.
     NoMorePairs,
-    /// Attempting to merge with a PSBT describing a different unsigned
+    /// Attempting to combine with a PSBT describing a different unsigned
     /// transaction.
     UnexpectedUnsignedTx {
         /// Expected
-        expected: Transaction,
+        expected: Box<Transaction>,
         /// Actual
-        actual: Transaction,
+        actual: Box<Transaction>,
     },
-    /// Unable to parse as a standard SigHash type.
-    NonStandardSigHashType(u32),
+    /// Unable to parse as a standard sighash type.
+    NonStandardSighashType(u32),
     /// Parsing errors from bitcoin_hashes
     HashParseError(hashes::Error),
     /// The pre-image must hash to the correponding psbt hash
@@ -68,12 +70,13 @@ pub enum Error {
         /// Hash-type
         hash_type: PsbtHash,
         /// Pre-image
-        preimage: Vec<u8>,
+        preimage: Box<[u8]>,
         /// Hash value
-        hash: Vec<u8>,
+        hash: Box<[u8]>,
     },
-    /// Data inconsistency/conflicting data during merge procedure
-    MergeConflict(String),
+    /// Conflicting data during combine procedure:
+    /// global extended public key has inconsistent key sources
+    CombineInconsistentKeySources(ExtendedPubKey),
     /// Serialization error in bitcoin consensus-encoded structures
     ConsensusEncoding,
 }
@@ -85,7 +88,7 @@ impl fmt::Display for Error {
             Error::InvalidProprietaryKey => write!(f, "non-proprietary key type found when proprietary key was expected"),
             Error::DuplicateKey(ref rkey) => write!(f, "duplicate key: {}", rkey),
             Error::UnexpectedUnsignedTx { expected: ref e, actual: ref a } => write!(f, "different unsigned transaction: expected {}, actual {}", e.txid(), a.txid()),
-            Error::NonStandardSigHashType(ref sht) => write!(f, "non-standard sighash type: {}", sht),
+            Error::NonStandardSighashType(ref sht) => write!(f, "non-standard sighash type: {}", sht),
             Error::InvalidMagic => f.write_str("invalid magic"),
             Error::InvalidSeparator => f.write_str("invalid separator"),
             Error::UnsignedTxHasScriptSigs => f.write_str("the unsigned transaction has script sigs"),
@@ -99,13 +102,14 @@ impl fmt::Display for Error {
                 // directly using debug forms of psbthash enums
                 write!(f, "Preimage {:?} does not match {:?} hash {:?}", preimage, hash_type, hash )
             }
-            Error::MergeConflict(ref s) => { write!(f, "Merge conflict: {}", s) }
+            Error::CombineInconsistentKeySources(ref s) => { write!(f, "combine conflict: {}", s) }
             Error::ConsensusEncoding => f.write_str("bitcoin consensus or BIP-174 encoding error"),
         }
     }
 }
 
-impl error::Error for Error {}
+#[cfg(feature = "std")]
+impl ::std::error::Error for Error {}
 
 #[doc(hidden)]
 impl From<hashes::Error> for Error {
